@@ -1,5 +1,19 @@
-# file MASS/polr.q
-# copyright (C) 1994-2005 W. N. Venables and B. D. Ripley
+# file MASS/R/polr.R
+# copyright (C) 1994-2008 W. N. Venables and B. D. Ripley
+# Use of transformed intercepts contributed by David Firth
+#
+#  This program is free software; you can redistribute it and/or modify
+#  it under the terms of the GNU General Public License as published by
+#  the Free Software Foundation; either version 2 or 3 of the License
+#  (at your option).
+#
+#  This program is distributed in the hope that it will be useful,
+#  but WITHOUT ANY WARRANTY; without even the implied warranty of
+#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#  GNU General Public License for more details.
+#
+#  A copy of the GNU General Public License is available at
+#  http://www.r-project.org/Licenses/
 #
 polr <- function(formula, data, weights, start, ..., subset,
                  na.action, contrasts = NULL, Hess = FALSE,
@@ -41,7 +55,7 @@ polr <- function(formula, data, weights, start, ..., subset,
         xx <- .polrY1*p1 - .polrY2*p2
         g2 <- - t(xx) %*% (wt/pr)
         g2 <- t(g2) %*% jacobian(theta)
-        if(all(pr) > 0) c(g1, g2) else rep(NA, pc+q)
+        if(all(pr > 0)) c(g1, g2) else rep(NA, pc+q)
     }
 
     m <- match.call(expand.dots = FALSE)
@@ -52,12 +66,12 @@ polr <- function(formula, data, weights, start, ..., subset,
                    cloglog = dgumbel, cauchit = dcauchy)
     if(is.matrix(eval.parent(m$data)))
         m$data <- as.data.frame(data)
-    m$start <- m$Hess <- m$method <- m$... <- NULL
+    m$start <- m$Hess <- m$method <- m$model <- m$... <- NULL
     m[[1]] <- as.name("model.frame")
     m <- eval.parent(m)
     Terms <- attr(m, "terms")
     x <- model.matrix(Terms, m, contrasts)
-    xint <- match("(Intercept)", colnames(x), nomatch=0)
+    xint <- match("(Intercept)", colnames(x), nomatch=0L)
     n <- nrow(x)
     pc <- ncol(x)
     cons <- attr(x, "contrasts") # will get dropped by subsetting
@@ -91,7 +105,7 @@ polr <- function(formula, data, weights, start, ..., subset,
                    "cloglog" = glm.fit(X, y1, wt, family = binomial("probit"), offset = offset),
                    "cauchit" = glm.fit(X, y1, wt, family = binomial("cauchit"), offset = offset))
         if(!fit$converged)
-            stop("attempt for find suitable starting values failed")
+            stop("attempt to find suitable starting values failed")
         coefs <- fit$coefficients
         if(any(is.na(coefs))) {
             warning("design appears to be rank-deficient, so dropping some coefs")
@@ -101,13 +115,17 @@ polr <- function(formula, data, weights, start, ..., subset,
             pc <- ncol(x)
         }
         spacing <- logit((1:q)/(q+1)) # just a guess
-        if(method != "logit") spacing <- spacing/1.7
+        if(method != "logistic") spacing <- spacing/1.7
         gammas <- -coefs[1] + spacing - spacing[q1]
         thetas <- c(gammas[1], log(diff(gammas)))
-        start <- c(coefs[-1], thetas)
+        s0 <- c(coefs[-1], thetas)
     } else if(length(start) != pc + q)
 	stop("'start' is not of the correct length")
-    res <- optim(start, fmin, gmin, method="BFGS", hessian = Hess, ...)
+    else {
+        s0 <- if(pc > 0) c(start[seq_len(pc+1)], diff(start[-seq_len(pc)]))
+        else c(start[1], diff(start))
+        }
+    res <- optim(s0, fmin, gmin, method="BFGS", hessian = Hess, ...)
     beta <- res$par[seq_len(pc)]
     theta <- res$par[pc + 1:q]
     zeta <- cumsum(c(theta[1],exp(theta[-1])))
@@ -147,7 +165,7 @@ print.polr <- function(x, ...)
 {
     if(!is.null(cl <- x$call)) {
         cat("Call:\n")
-        dput(cl)
+        dput(cl, control=NULL)
     }
     if(length(coef(x))) {
         cat("\nCoefficients:\n")
@@ -159,7 +177,7 @@ print.polr <- function(x, ...)
     print(x$zeta, ...)
     cat("\nResidual Deviance:", format(x$deviance, nsmall=2), "\n")
     cat("AIC:", format(x$deviance + 2*x$edf, nsmall=2), "\n")
-    if(nchar(mess <- naprint(x$na.action))) cat("(", mess, ")\n", sep="")
+    if(nzchar(mess <- naprint(x$na.action))) cat("(", mess, ")\n", sep="")
     if(x$convergence > 0)
         cat("Warning: did not converge as iteration limit reached\n")
     invisible(x)
@@ -168,10 +186,10 @@ print.polr <- function(x, ...)
 vcov.polr <- function(object, ...)
 {
     if(is.null(object$Hessian)) {
-        cat("\nRe-fitting to get Hessian\n\n")
-	flush.console
+        message("\nRe-fitting to get Hessian\n")
+	utils::flush.console()
         object <- update(object, Hess=TRUE,
-                         start=c(object$coef, object$zeta))
+                         start=c(object$coefficients, object$zeta))
     }
     structure(ginv(object$Hessian), dimnames = dimnames(object$Hessian))
 
@@ -217,13 +235,13 @@ print.summary.polr <- function(x, digits = x$digits, ...)
 {
     if(!is.null(cl <- x$call)) {
         cat("Call:\n")
-        dput(cl)
+        dput(cl, control=NULL)
     }
-    coef <- format(round(x$coef, digits=digits))
+    coef <- format(round(x$coefficients, digits=digits))
     pc <- x$pc
     if(pc > 0) {
         cat("\nCoefficients:\n")
-        print(x$coef[seq_len(pc), , drop=FALSE], quote = FALSE, ...)
+        print(x$coefficients[seq_len(pc), , drop=FALSE], quote = FALSE, ...)
     } else {
         cat("\nNo coefficients\n")
     }
@@ -231,7 +249,7 @@ print.summary.polr <- function(x, digits = x$digits, ...)
     print(coef[(pc+1):nrow(coef), , drop=FALSE], quote = FALSE, ...)
     cat("\nResidual Deviance:", format(x$deviance, nsmall=2), "\n")
     cat("AIC:", format(x$deviance + 2*x$edf, nsmall=2), "\n")
-    if(nchar(mess <- naprint(x$na.action))) cat("(", mess, ")\n", sep="")
+    if(nzchar(mess <- naprint(x$na.action))) cat("(", mess, ")\n", sep="")
     if(!is.null(correl <- x$correlation)) {
         cat("\nCorrelation of Coefficients:\n")
         ll <- lower.tri(correl)
@@ -255,11 +273,11 @@ predict.polr <- function(object, newdata, type=c("class","probs"), ...)
         if (!is.null(cl <- attr(Terms, "dataClasses")))
             .checkMFClasses(cl, m)
         X <- model.matrix(Terms, m, contrasts = object$contrasts)
-        xint <- match("(Intercept)", colnames(X), nomatch=0)
+        xint <- match("(Intercept)", colnames(X), nomatch=0L)
         if(xint > 0) X <- X[, -xint, drop=FALSE]
         n <- nrow(X)
         q <- length(object$zeta)
-        eta <- drop(X %*% object$coef)
+        eta <- drop(X %*% object$coefficients)
         pfun <- switch(object$method, logistic = plogis, probit = pnorm,
                        cloglog = pgumbel, cauchit = pcauchy)
         cumpr <- matrix(pfun(matrix(object$zeta, n, q, byrow=TRUE) - eta), , q)
@@ -387,7 +405,7 @@ polr.fit <- function(x, y, wt, start, offset, method)
         xx <- .polrY1*p1 - .polrY2*p2
         g2 <- - t(xx) %*% (wt/pr)
         g2 <- t(g2) %*% jacobian(theta)
-        if(all(pr) > 0) c(g1, g2) else rep(NA, pc+q)
+        if(all(pr > 0)) c(g1, g2) else rep(NA, pc+q)
     }
 
     pfun <- switch(method, logistic = plogis, probit = pnorm,
@@ -403,7 +421,10 @@ polr.fit <- function(x, y, wt, start, offset, method)
     Y <- matrix(0, n, q)
     .polrY1 <- col(Y) == y
     .polrY2 <- col(Y) == y - 1
-    res <- optim(start, fmin, gmin, method="BFGS")
+    # pc could be 0.
+    s0 <- if(pc > 0) c(start[seq_len(pc+1)], diff(start[-seq_len(pc)]))
+    else c(start[1], diff(start))
+    res <- optim(s0, fmin, gmin, method="BFGS")
     beta <- res$par[seq_len(pc)]
     theta <- res$par[pc + 1:q]
     zeta <- cumsum(c(theta[1],exp(theta[-1])))
@@ -435,7 +456,7 @@ profile.polr <- function(fitted, which = 1:p, alpha = 0.01,
     if(length(W) == 0) W <- rep(1, n)
     OriginalDeviance <- deviance(fitted)
     X <- model.matrix(fitted)[, -1, drop=FALSE] # drop intercept
-    zmax <- sqrt(qchisq(1 - alpha/2, p))
+    zmax <- sqrt(qchisq(1 - alpha, 1))
     profName <- "z"
     prof <- vector("list", length=length(which))
     names(prof) <- Pnames[which]
@@ -447,8 +468,8 @@ profile.polr <- function(fitted, which = 1:p, alpha = 0.01,
         pi <- Pnames[i]
         for(sgn in c(-1, 1)) {
             if(trace) {
-                cat("\nParameter:", pi, c("down", "up")[(sgn + 1)/2 + 1], "\n")
-                flush.console()
+                message("\nParameter:", pi, c("down", "up")[(sgn + 1)/2 + 1])
+                utils::flush.console()
             }
             step <- 0
             z <- 0
@@ -483,9 +504,9 @@ confint.polr <- function(object, parm, level = 0.95, trace = FALSE, ...)
 {
     pnames <- names(coef(object))
     if(missing(parm)) parm <- seq_along(pnames)
-    else if(is.character(parm))  parm <- match(parm, pnames, nomatch = 0)
-    cat("Waiting for profiling to be done...\n")
-    flush.console()
+    else if(is.character(parm))  parm <- match(parm, pnames, nomatch = 0L)
+    message("Waiting for profiling to be done...")
+    utils::flush.console()
     object <- profile(object, which = parm, alpha = (1. - level)/4.,
                       trace = trace)
     confint(object, parm=parm, level=level, trace=trace, ...)
@@ -496,7 +517,7 @@ confint.profile.polr <-
 {
     of <- attr(object, "original.fit")
     pnames <- names(coef(of))
-    if(is.character(parm))  parm <- match(parm, pnames, nomatch = 0)
+    if(is.character(parm))  parm <- match(parm, pnames, nomatch = 0L)
     a <- (1-level)/2
     a <- c(a, 1-a)
     pct <- paste(round(100*a, 1), "%")
@@ -512,3 +533,6 @@ confint.profile.polr <-
     }
     drop(ci)
 }
+
+logLik.polr <- function(object, ...)
+    structure(-0.5 * object$deviance, df = object$edf, class = "logLik")
